;***************************************
;***  component color video          ***
;***     for pic16F84A               ***
;***  draw color texture signal      ***
;***     on component video.         ***
;***  external 20MHz osc.            ***
;***  assign RA0-RA2 to B-R-G signal ***
;***  assign RB0 to Sync signal      ***
;***             m.suga              ***
;***             created 2008.5.17   ***
;***************************************


#include <P16F84A.INC>
	__CONFIG	_CP_OFF & _PWRTE_OFF & _WDT_OFF & _HS_OSC


	;**********************************
	;*****  DEFINE REGISTER FILE  *****
	;**********************************

line	EQU	0Ch
l_count1	EQU	0Dh
l_count2	EQU 0Eh
out_data	EQU	0Fh
color_num	EQU 10h
color_count	EQU 11h

out_image1	EQU 12h
out_image2	EQU 13h
out_image3	EQU 14h
out_image4	EQU 15h
out_image5	EQU 16h
out_image6	EQU 17h
out_image7	EQU 18h
out_image8	EQU 19h

tmp			EQU 1Ah


	GOTO	init


	;************************
	;*****  INITIALIZE  *****
	;************************

init:
	BSF		STATUS,RP0			;SET BANK1

	MOVLW	B'00000000'			;SET W AS ALL OUTPUT MODE
	MOVWF	TRISA				;INIT TRISA

	MOVLW	B'00000000'			;SET W AS ALL OUTPUT MODE
	MOVWF	TRISB				;INIT TRISB

	bcf		STATUS,RP0			;SET BANK0


	MOVLW	d'0'
	MOVWF	line



	;***********************
	;*****  MAIN LOOP  *****
	;***********************

mainloop:

	call	EQ_PULSE		;EQ PULSE
	call	EQ_PULSE		;EQ PULSE
	call	EQ_PULSE		;EQ PULSE
	call	V_SYNC			;V SYNC
	call	V_SYNC			;V SYNC
	call	V_SYNC			;V SYNC
	call	EQ_PULSE		;EQ PULSE
	call	EQ_PULSE		;EQ PULSE
	call	EQ_PULSE		;EQ PULSE


LOOP1:
	call	BLANK_DRAW		;draw blank line

	incf	line,F			;line counter +
	movlw	d'28'
	subwf	line,W			;compare line counter with 28
	btfsc	STATUS,C		;skip if under 28
							; (closed caption 12 + blank line 16)
	goto	LOOP2
	goto	LOOP1			;loop


LOOP2:
	movlw	0xff			;spread pattern
	movwf	out_data
	call	COLOR_DRAW		;draw color lines

	incf	line,F			;line counter +
	movlw	d'78'
	subwf	line,W			;compare line counter with 78 (28+50)
	btfsc	STATUS,C		;skip if under 78

	goto	LOOP3
	goto	LOOP2			;loop


LOOP3:
	movlw	0xf0			;fat line pattern
	movwf	out_data
	call	COLOR_DRAW		;draw color lines

	incf	line,F			;line counter +
	movlw	d'128'
	subwf	line,W			;compare line counter with 128 (28+50+50)
	btfsc	STATUS,C		;skip if under 128

	goto	LOOP4
	goto	LOOP3			;loop


LOOP4:
	movlw	0xcc			;middle line pattern
	movwf	out_data
	call	COLOR_DRAW		;draw color lines

	incf	line,F			;line counter +
	movlw	d'178'
	subwf	line,W			;compare line counter with 178 (28+50+50+50)
	btfsc	STATUS,C		;skip if under 178

	goto	LOOP5
	goto	LOOP4			;loop


LOOP5:
	movlw	0xaa			;narrow line pattern
	movwf	out_data
	call	COLOR_DRAW		;draw color lines

	incf	line,F			;line counter +
	movlw	d'228'
	subwf	line,W			;compare line counter with 228 (28+50+50+50+50)
	btfsc	STATUS,C		;skip if under 228
	goto	LOOP6
	goto	LOOP5			;loop


LOOP6:
	call	BLANK_DRAW		;draw blank line

	incf	line,F			;line counter +
	movlw	d'253'
	subwf	line,W			;compare line 253 (262H-9H)
	btfsc	STATUS,C		;skip if under 253
	goto	CHK1
	goto	LOOP6			;loop


CHK1:

	MOVLW	d'0'
	MOVWF	line

	GOTO	mainloop





	;************************
	;*****  SUB RUTINE  *****
	;************************



	;*****                   *****
	;*****  OUTPUT EQ PULSE  *****
	;*****                   *****

EQ_PULSE:

	;*****  output sync signal  *****
	clrf	PORTA			;color non
	bSf		PORTB,0			;sync on

	;*****  2.4us  *****
	movlw	d'3'			;0.2us
	movwf	l_count1
EQ_LOOP1:
	decfsz	l_count1,F
	goto	EQ_LOOP1		;0.2*3 *3 = 1.8
							;1.8+0.2 = 2.0us
	nop
	nop						;+0.4us


	;*****  outpout black signal  *****
	bcf		PORTB,0			;sync off

	;*****  28.4us  *****
	movlw	d'47'			;0.2us
	movwf	l_count1
EQ_LOOP2:
	decfsz	l_count1,F
	goto	EQ_LOOP2		;0.2*3 *47 = 28.2
							;28.2+0.2 = 28.4us


	;*****  output sync signal  *****
	bSf		PORTB,0			;sync on

	;*****  2.4us  *****
	movlw	d'3'			;0.2us
	movwf	l_count1
EQ_LOOP3:
	decfsz	l_count1,F
	goto	EQ_LOOP3		;0.2*3 *3 = 1.8
							;1.8+0.2 = 2.0us
	nop
	nop						;+0.4us


	;*****  outpout black signal  *****
	bcf		PORTB,0			;sync off

	;*****  28.4us  *****
	movlw	d'47'			;0.2us
	movwf	l_count1
EQ_LOOP4:
	decfsz	l_count1,F
	goto	EQ_LOOP4		;0.2*3 *47 = 28.2
							;28.2+0.2 = 28.4us


	return					;return to main




	;*****                 *****
	;*****  OUTPUT V SYNC  *****
	;*****                 *****

V_SYNC:

	;*****  output sync signal  *****
	clrf	PORTA			;color non
	bSf		PORTB,0			;sync on

	;*****  26.2us  *****
	movlw	d'43'			;0.2us
	movwf	l_count1
VS_LOOP1:
	decfsz	l_count1,F
	goto	VS_LOOP1		;0.2*3 *43 = 25.8
							;25.8+0.2 = 26.0us
	nop						;+0.2us


	;*****  outpout black signal  *****
	bcf		PORTB,0			;sync off

	;*****  4.6us  *****
	movlw	d'7'				;0.2us
	movwf	l_count1
VS_LOOP2:
	decfsz	l_count1,F
	goto	VS_LOOP2		;0.2*3 *7 = 4.2
							;4.2+0.2 = 4.4us
	nop						;+0.2us


	;*****  output sync signal  *****
	bSf		PORTB,0			;sync on

	;*****  26.2us  *****
	movlw	d'43'			;0.2us
	movwf	l_count1
VS_LOOP3:
	decfsz	l_count1,F
	goto	VS_LOOP3		;0.2*3 *43 = 25.8
							;265.8+0.2 = 26.0us
	nop						;+0.2us


	;*****  outpout black signal  *****
	bcf		PORTB,0			;sync off

	;*****  4.6us  *****
	movlw	d'7'			;0.2us
	movwf	l_count1
VS_LOOP4:
	decfsz	l_count1,F
	goto	VS_LOOP4		;0.2*3 *7 = 4.2
							;4.2+0.2 = 4.4us
	nop						;+0.2us


	return					;return to main





	;*****                   *****
	;*****  DRAW BLANK LINE  *****
	;*****                   *****

BLANK_DRAW:
	;*****  output sync signal  *****
	clrf	PORTA			;color non
	bSf		PORTB,0			;sync on

	;*****  4.6us  *****
	movlw	d'7'				;0.2us
	movwf	l_count1
BD_LOOP1:
	decfsz	l_count1,F
	goto	BD_LOOP1		;0.2*3 *7 = 4.2
							;4.2+0.2 = 4.4us
	nop						;+0.2us


	;*****  output black signal  *****
	bcf		PORTB,0			;sync off

	;*****  56.0us  *****
	movlw	d'93'			;0.2us
	movwf	l_count1
BD_LOOP2:
	decfsz	l_count1,F
	goto	BD_LOOP2		;0.2*3 *93 = 55.8
							;55.8+0.2 = 56.0us


	return						;return to main





	;*****                   *****
	;*****  draw color line  *****
	;*****                   *****

COLOR_DRAW:
	;*****  output sync signal  *****
	clrf	PORTA			;color non
	bSf		PORTB,0			;sync on

	;*****  4.6us  *****
	movlw	d'7'			;0.2us
	movwf	l_count1
DR_LOOP1:
	decfsz	l_count1,F
	goto	DR_LOOP1		;0.2*3 *7 = 4.2
							;4.2+0.2 = 4.4us
	nop						;+0.2us

	;*****  output black signal  *****
	bcf		PORTB,0			;sync off



	;*****  decoding image data  *****

	movlw	b'00000111'
	movwf	tmp


	rlf		out_data,F
	btfss	STATUS,C
	goto	DR_CHK1

	movf	tmp,W
	movwf	out_image1
	goto	DR_CHK2

DR_CHK1:
	clrf	out_image1
	nop
	nop

DR_CHK2:


	rlf		out_data,F
	btfss	STATUS,C
	goto	DR_CHK3

	movf	tmp,W
	movwf	out_image2
	goto	DR_CHK4

DR_CHK3:
	clrf	out_image2
	nop
	nop

DR_CHK4:


	rlf		out_data,F
	btfss	STATUS,C
	goto	DR_CHK5

	movf	tmp,W
	movwf	out_image3
	goto	DR_CHK6

DR_CHK5:
	clrf	out_image3
	nop
	nop

DR_CHK6:


	rlf		out_data,F
	btfss	STATUS,C
	goto	DR_CHK7

	movf	tmp,W
	movwf	out_image4
	goto	DR_CHK8

DR_CHK7:
	clrf	out_image4
	nop
	nop

DR_CHK8:


	rlf		out_data,F
	btfss	STATUS,C
	goto	DR_CHK9

	movf	tmp,W
	movwf	out_image5
	goto	DR_CHK10

DR_CHK9:
	clrf	out_image5
	nop
	nop

DR_CHK10:


	rlf		out_data,F
	btfss	STATUS,C
	goto	DR_CHK11

	movf	tmp,W
	movwf	out_image6
	goto	DR_CHK12

DR_CHK11:
	clrf	out_image6
	nop
	nop

DR_CHK12:


	rlf		out_data,F
	btfss	STATUS,C
	goto	DR_CHK13

	movf	tmp,W
	movwf	out_image7
	goto	DR_CHK14

DR_CHK13:
	clrf	out_image7
	nop
	nop

DR_CHK14:


	rlf		out_data,F
	btfss	STATUS,C
	goto	DR_CHK15

	movf	tmp,W
	movwf	out_image8
	goto	DR_CHK16

DR_CHK15:
	clrf	out_image8
	nop
	nop

DR_CHK16:
							;66*0.2 = 13.2us




	;*****  output color data  *****

	movlw	d'7'
	movwf	color_count

DR_LOOP2:
	movf	color_count,W
	andwf	out_image1,W
	movwf	PORTA

	movf	color_count,W
	andwf	out_image2,W
	movwf	PORTA

	movf	color_count,W
	andwf	out_image3,W
	movwf	PORTA

	movf	color_count,W
	andwf	out_image4,W
	movwf	PORTA

	movf	color_count,W
	andwf	out_image5,W
	movwf	PORTA

	movf	color_count,W
	andwf	out_image6,W
	movwf	PORTA

	movf	color_count,W
	andwf	out_image7,W
	movwf	PORTA

	movf	color_count,W
	andwf	out_image8,W
	movwf	PORTA


	decfsz	color_count,F
	goto	DR_LOOP2		;loop 7times (7 colors)

							;27*7*0.2 +0.2 = 38.0us



	;*****  output black code  *****
	bcf		PORTB,0			;sync off
	clrf	PORTA			;color non


	;*****  5.6us  *****
	movlw	d'9'			;0.2us
	movwf	l_count1
DR_LOOP3:
	decfsz	l_count1,F
	goto	DR_LOOP3		;0.2*3 *9 = 5.4
							;5.4+0.2 = 5.6us


	return					;return to main






	GOTO	$
	;---endproc---

	END
